#ifndef WXFMODPLAYER_H_INCLUDED
#define WXFMODPLAYER_H_INCLUDED

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
 #include <wx/wx.h>
#endif
#include "fmodex/fmod.hpp"

#define FMP_BUTTON_NONE 0
#define FMP_BUTTON_PLAY 1
#define FMP_BUTTON_STOP 2
#define FMP_BUTTON_PREV 4
#define FMP_BUTTON_NEXT 8
#define FMP_BUTTON_ALL 15

extern const wxEventType wxEVT_FMP_NEXTFILE;
extern const wxEventType wxEVT_FMP_PREVFILE;
extern const wxEventType wxEVT_FMP_FINISHED;

#define EVT_FMP_NEXTFILE(id, fn) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wxEVT_FMP_NEXTFILE, id, wxID_ANY, \
        (wxObjectEventFunction)(wxEventFunction) wxStaticCastEvent( wxCommandEventFunction, &fn ), \
        (wxObject *) NULL \
    ),
#define EVT_FMP_PREVFILE(id, fn) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wxEVT_FMP_PREVFILE, id, wxID_ANY, \
        (wxObjectEventFunction)(wxEventFunction) wxStaticCastEvent( wxCommandEventFunction, &fn ), \
        (wxObject *) NULL \
    ),
#define EVT_FMP_FINISHED(id, fn) \
    DECLARE_EVENT_TABLE_ENTRY( \
        wxEVT_FMP_FINISHED, id, wxID_ANY, \
        (wxObjectEventFunction)(wxEventFunction) wxStaticCastEvent( wxCommandEventFunction, &fn ), \
        (wxObject *) NULL \
    ),

class wxFModPlayer : public wxPanel
{
public:
    wxFModPlayer(wxWindow* parent,wxWindowID id);
    virtual ~wxFModPlayer();
    void EnableButtons(int ButtonsMask=FMP_BUTTON_ALL);
    bool Initialize();
    bool SetStreamDatas(void* datas,size_t length,const wxString& DispName=_T(""));
    void SetDisplayName(const wxString& dispname);
    bool StartPlaying(bool restart=true);
    void StopPlaying();
    bool IsPlaying();
    bool IsPaused();
protected:
private:
    wxStaticBox* stbMain;
    wxStaticText* sttStrmInfo,* sttPlayInfo;
    wxSlider* sldProgress,* sldVolume;
    wxBitmapButton* btnPlay,* btnStop,* btnNext,* btnPrev;
    wxTimer* timer;
    bool bIsInitialized,bDatasOk,bIsChangingPos,bIsPlaying,bIsPaused;
    unsigned int iCurLengthMs;
    char *currbuff;
    FMOD::System *fm_system;
    FMOD::Sound *fm_sound;
    FMOD::Channel *fm_channel;
    FMOD_RESULT fm_result;
    void OnTimer(wxTimerEvent& event);
    void OnBtnPlayClicked(wxCommandEvent& event);
    void OnBtnStopClicked(wxCommandEvent& event);
    void OnBtnPrevClicked(wxCommandEvent& event);
    void OnBtnNextClicked(wxCommandEvent& event);
    void OnChangeVolume(wxScrollEvent& event);
    void OnChangePosition(wxScrollEvent& event);
    DECLARE_EVENT_TABLE();
};

#endif // WXFMODPLAYER_H_INCLUDED
